from enum import IntEnum


class WorkMode(IntEnum):
    self_use = 0
    feedin_priority = 1
    back_up_mode = 2
    manual_mode = 3
    peak_shaving = 4
    tou = 5
    vpp = 16

    def __str__(self):
        return {
            self.self_use: "self_use",
            self.feedin_priority: "feedin_priority",
            self.back_up_mode: "back_up_mode",
            self.manual_mode: "manual_mode",
            self.peak_shaving: "peak_shaving",
            self.vpp: "vpp",
        }.get(self)


class ManualMode(IntEnum):
    forced_charging = 3
    forced_discharging = 4
    stop_charging_discharging = 5

    def __str__(self):
        return {
            self.forced_charging: "forced_charging",
            self.forced_discharging: "forced_discharging",
            self.stop_charging_discharging: "stop_charging_and_discharging",
        }.get(self)


class VppMode(IntEnum):
    power_control_mode = 1
    electric_quantity_target_control_mode = 2
    soc_target_control_mode = 3
    push_power_positive_negative_mode = 4
    push_power_zero_mode = 5
    self_consume_charge_discharge_mode = 6
    self_consume_charge_only_mode = 7
    pv_bat_individual_setting_duration_mode = 8
    pv_bat_individual_setting_target_soc_mode = 9

    def __str__(self):
        return {
            self.power_control_mode: "power_control_mode",
            self.electric_quantity_target_control_mode: "electric_quantity_target_control_mode",
            self.soc_target_control_mode: "soc_target_control_mode",
            self.push_power_positive_negative_mode: "push_power_positive_negative_mode",
            self.push_power_zero_mode: "push_power_zero_mode",
            self.self_consume_charge_discharge_mode: "self_consume_charge_discharge_mode",
            self.self_consume_charge_only_mode: "self_consume_charge_only_mode",
            self.pv_bat_individual_setting_duration_mode: "pv_bat_individual_setting_duration_mode",
            self.pv_bat_individual_setting_target_soc_mode: "pv_bat_individual_setting_target_soc_mode",
        }.get(self)



from unittest import TestCase

from solax_py_library.smart_scene.core.service.check import (
    condition_param_check,
    action_param_check,
)
from solax_py_library.smart_scene.types.action import (
    SmartSceneAction,
    ActionType,
    SystemActionType,
    SystemActionItemData,
    SystemActionChildData,
)
from solax_py_library.smart_scene.types.condition import (
    SmartSceneCondition,
    ConditionItem,
    ConditionType,
    SystemConditionItemData,
    LogicFunc,
    SystemConditionType,
    ConditionItemChildData,
    CabinetConditionItemData,
    CabinetConditionType,
    ConditionFunc,
)


class TestCheckFunc(TestCase):
    def test_check_condition_func(self):
        con = SmartSceneCondition(
            operation=LogicFunc.AND,
            value=[
                ConditionItem(
                    type=ConditionType.systemCondition,
                    data=[
                        SystemConditionItemData(
                            childType=SystemConditionType.systemSoc,
                            childData=ConditionItemChildData(
                                data=[101], function=ConditionFunc.EQ
                            ),
                        ),
                    ],
                    cabinet=None,
                ),
                ConditionItem(
                    type=ConditionType.cabinet,
                    data=[
                        CabinetConditionItemData(
                            childType=CabinetConditionType.cabinetSoc,
                            childData=ConditionItemChildData(
                                data=[4], function=ConditionFunc.EQ
                            ),
                        )
                    ],
                    cabinet=["1", "2"],
                ),
            ],
        )
        ret = condition_param_check(con, ctx={"soc_low_limit": 10})
        assert ret is not None

    def test_check_action_func(self):
        action = [
            SmartSceneAction(
                type=ActionType.system,
                data=[
                    SystemActionItemData(
                        childType=SystemActionType.importControl,
                        childData=SystemActionChildData(data=[1, -10]),
                    )
                ],
            )
        ]
        ret = action_param_check(
            actions=action,
            ctx={
                "cabinet_type": 3,
                "total_power": 1,
                "total_power_except_solar_inv": 1,
                "total_energy": 1,
            },
        )
        assert ret is not None



import os
from enum import IntEnum
from typing import Any, Optional

from pydantic import BaseModel


class FTPFileType(IntEnum):
    CSV = 1

    @classmethod
    def get_file_suffix(cls, value):
        return {
            FTPFileType.CSV: ".csv",
        }.get(value)


class FTPData(BaseModel):
    file_type: FTPFileType
    file_name: str
    data: Any

    def build_full_path(self, remote_path) -> str:
        return os.path.join(remote_path, self.file_name)


class FTPServiceConfig(BaseModel):
    host: str
    port: int
    user: Optional[str]
    password: Optional[str]
    remote_path: str


class FTPParsedData(BaseModel):
    file_name: str
    file_path: str



import csv
import tempfile
from typing import Any, Union, List, Dict

from solax_py_library.upload.core.data_adapter.base import BaseDataAdapter


class CSVDataAdapter(BaseDataAdapter):
    @classmethod
    def parse_data(cls, data: Union[List[List[Any]], List[Dict[str, Any]]]):
        with tempfile.NamedTemporaryFile(
            delete=False, mode="w", newline="", encoding="utf-8-sig"
        ) as temp_file:
            if isinstance(data[0], list):
                writer = csv.writer(temp_file)
                writer.writerows(data)
            else:
                headers = list(data[0].keys())
                writer = csv.DictWriter(temp_file, fieldnames=headers)
                writer.writeheader()
                writer.writerows(data)

            temp_file_path = temp_file.name
            temp_file.seek(0)

        return temp_file_path



from datetime import datetime, timedelta


def trans_str_time_to_index(now_time, minute=15):
    """将时间按照minute切换为索引，时间格式为 %H-%M"""
    time_list = [int(i) for i in now_time.split(":")]
    time_int = time_list[0] * 4 + time_list[1] // minute
    return time_int


def get_left_timestamp(now_time, minute=15):
    return now_time // (60 * minute * 1000) * (60 * minute * 1000)


def get_highest_or_lowest_value(start_time, end_time, hours, price_list, reverse=False):
    start_index = trans_str_time_to_index(start_time)
    end_index = trans_str_time_to_index(end_time)
    arr = price_list[start_index:end_index]
    if None in arr:
        return False
    indices = list(range(end_index - start_index))
    sorted_indices = sorted(indices, key=lambda i: arr[i], reverse=reverse)
    return sorted_indices[: int(hours * 4)], start_index


def get_rounded_times():
    """
    返回距离当前时间最近的15min的整点时间以及后一整点5min时间（天气是预测未来15min的，也就是在00:00时，只能拿到00:15的数据）
    """
    now = datetime.now()
    # 确定当前时间所属的15分钟区间
    index_1 = now.minute // 15
    index_2 = now.minute % 15
    left_time = now.replace(minute=15 * index_1, second=0, microsecond=0)
    right_time = left_time + timedelta(minutes=15)
    if index_2 < 8:
        nearest_time = left_time
    else:
        nearest_time = right_time
    return datetime.strftime(nearest_time, "%Y-%m-%d %H:%M:%S"), datetime.strftime(
        right_time, "%Y-%m-%d %H:%M:%S"
    )


def hour_to_today_timestamp(hour: int):
    if hour != 24:
        target_date = datetime.now()
        hour = hour
    else:
        target_date = datetime.now() + timedelta(days=1)
        hour = 0
    target = datetime.strptime(
        f"{target_date.year}-{target_date.month}-{target_date.day} {hour}:00:00",
        "%Y-%m-%d %H:%M:%S",
    )
    return int(target.timestamp())



from typing import List

from pydantic import BaseModel, Field

from solax_py_library.device.types.device import DeviceType


class CabinetValue(BaseModel):
    soc: int = None
    do5: int = None
    cabinet_alarm: List[bool] = Field(default_factory=lambda: [False, False, False])
    pcs_alarm: List[bool] = Field(default_factory=lambda: [False, False, False])
    io_alarm: List[bool] = Field(default_factory=lambda: [False, False, False])
    bms_alarm: List[bool] = Field(default_factory=lambda: [False, False, False])
    air_alarm: List[bool] = Field(default_factory=lambda: [False, False, False])
    liquid_alarm: List[bool] = Field(default_factory=lambda: [False, False, False])
    environment_monitoring_alarm: List[bool] = Field(
        default_factory=lambda: [False, False, False]
    )

    def alarm_info(self, device_type):
        if device_type == DeviceType.ESS_TYPE:
            return self.cabinet_alarm
        elif device_type == DeviceType.PCS_TYPE:
            return self.pcs_alarm
        elif device_type == DeviceType.IO_TYPE:
            return self.io_alarm
        elif device_type == DeviceType.BMS_TYPE:
            return self.bms_alarm
        elif device_type == DeviceType.AIRCONDITIONER_TYPE:
            return self.air_alarm
        elif device_type == DeviceType.COLD_TYPE:
            return self.liquid_alarm
        elif device_type == DeviceType.ENVIRONMENT_MONITORING_TYPE:
            return self.environment_monitoring_alarm
        return None



import asyncio
import unittest

from solax_py_library.upload.api.service import upload
from solax_py_library.upload.core.upload_service import FTPUploadService
from solax_py_library.upload.exceptions import ConnectError, LoginError
from solax_py_library.upload.types.client import UploadType, UploadData
from solax_py_library.upload.types.ftp import FTPFileType


class FTPTest(unittest.TestCase):
    def test_connect(self):
        ftp_config = {
            "host": "10.1.31.181",  # 测试host
            "port": 21,
            "user": "solax",
            "password": "123456",
            "remote_path": "/xixi",
        }
        ftp = FTPUploadService(**ftp_config)
        ftp.connect()

    def test_connect_error_1(self):
        ftp_config = {
            "host": "10.1.31.182",  # 测试host
            "port": 21,
            "user": "solax",
            "password": "123456",
            "remote_path": "/xixi",
        }
        ftp = FTPUploadService(**ftp_config)
        try:
            ftp.connect()
        except ConnectError:
            ...

    def test_connect_error_2(self):
        ftp_config = {
            "host": "10.1.31.181",  # 测试host
            "port": 21,
            "user": "solax123",
            "password": "123456",
            "remote_path": "/xixi",
        }
        ftp = FTPUploadService(**ftp_config)
        try:
            ftp.connect()
        except LoginError:
            ...

    def test_ftp_upload_to_windows(self):
        ftp_config = {
            "host": "10.1.31.181",  # 测试host
            "port": 21,
            "user": "solax",
            "password": "123456",
            "remote_path": "嘻嘻",
        }
        asyncio.run(
            upload(
                upload_type=UploadType.FTP,
                configuration=ftp_config,
                upload_data=UploadData(
                    upload_type=UploadType.FTP,
                    data=dict(
                        file_type=FTPFileType.CSV,
                        file_name="中文",
                        data=[
                            {
                                "EMS1000序列号": "XMG11A011L",
                                "EMS1000本地时间": "2025-02-11 15:39:10",
                                "EMS1000版本号": "V007.11.1",
                                "电站所在国家和地区": None,
                                "电站所在当前时区": None,
                                "电站系统类型": None,
                            }
                        ],
                    ),
                ),
            )
        )

    def test_ftp_upload_to_linux(self):
        ftp_config = {
            "host": "920729yofx76.vicp.fun",  # 测试host
            "port": 59477,
            "user": "test",
            "password": "test123456",
            "remote_path": "test",
        }
        asyncio.run(
            upload(
                upload_type=UploadType.FTP,
                configuration=ftp_config,
                upload_data=UploadData(
                    upload_type=UploadType.FTP,
                    data=dict(
                        file_type=FTPFileType.CSV,
                        file_name="中文",
                        data=[
                            {
                                "EMS1000序列号": "XMG11A011L",
                                "EMS1000本地时间": "2025-02-11 15:39:10",
                                "EMS1000版本号": "V007.11.1",
                                "电站所在国家和地区": None,
                                "电站所在当前时区": None,
                                "电站系统类型": None,
                            }
                        ],
                    ),
                ),
            )
        )



from .client import UploadType, UploadData
from .ftp import FTPData, FTPParsedData, FTPFileType, FTPServiceConfig

__all__ = [
    "FTPData",
    "FTPParsedData",
    "FTPServiceConfig",
    "UploadType",
    "UploadData",
    "FTPFileType",
]
